/****************************************************************************
**
** Copyright (C) 2012 Nokia Corporation and/or its subsidiary(-ies).
** Contact: http://www.qt-project.org/
**
** This file is part of the QtQml module of the Qt Toolkit.
**
** $QT_BEGIN_LICENSE:LGPL$
** GNU Lesser General Public License Usage
** This file may be used under the terms of the GNU Lesser General Public
** License version 2.1 as published by the Free Software Foundation and
** appearing in the file LICENSE.LGPL included in the packaging of this
** file. Please review the following information to ensure the GNU Lesser
** General Public License version 2.1 requirements will be met:
** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
**
** In addition, as a special exception, Nokia gives you certain additional
** rights. These rights are described in the Nokia Qt LGPL Exception
** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU General
** Public License version 3.0 as published by the Free Software Foundation
** and appearing in the file LICENSE.GPL included in the packaging of this
** file. Please review the following information to ensure the GNU General
** Public License version 3.0 requirements will be met:
** http://www.gnu.org/copyleft/gpl.html.
**
** Other Usage
** Alternatively, this file may be used in accordance with the terms and
** conditions contained in a signed written agreement between you and Nokia.
**
**
**
**
**
**
** $QT_END_LICENSE$
**
****************************************************************************/

#ifndef QQUICKSHADEREFFECTNODE_P_H
#define QQUICKSHADEREFFECTNODE_P_H

#include <QtQuick/qsgnode.h>
#include <QtQuick/qsgmaterial.h>
#include <QtQuick/qsgtextureprovider.h>
#include <QtQuick/qquickitem.h>
#include <private/qtquickglobal_p.h>

#include <QtCore/qsharedpointer.h>
#include <QtCore/qpointer.h>

QT_BEGIN_HEADER

QT_BEGIN_NAMESPACE

struct QQuickShaderEffectMaterialKey {
    enum ShaderType
    {
        VertexShader,
        FragmentShader,
        ShaderTypeCount
    };

    QByteArray sourceCode[ShaderTypeCount];
    const char *className;

    bool operator == (const QQuickShaderEffectMaterialKey &other) const;
};

uint qHash(const QQuickShaderEffectMaterialKey &key);


class QQuickCustomMaterialShader;
class QQuickShaderEffectNode;
class Q_QUICK_PRIVATE_EXPORT QQuickShaderEffectMaterial : public QSGMaterial
{
public:
    struct UniformData
    {
        enum SpecialType { None, Sampler, Opacity, Matrix };

        QByteArray name;
        QVariant value;
        SpecialType specialType;
    };

    enum CullMode
    {
        NoCulling,
        BackFaceCulling,
        FrontFaceCulling
    };

    explicit QQuickShaderEffectMaterial(QQuickShaderEffectNode *node = 0);
    virtual QSGMaterialType *type() const;
    virtual QSGMaterialShader *createShader() const;
    virtual int compare(const QSGMaterial *other) const;

    QVector<QByteArray> attributes;
    QVector<UniformData> uniforms[QQuickShaderEffectMaterialKey::ShaderTypeCount];
    QVector<QPair<QByteArray, QSGTextureProvider *> > textureProviders;
    CullMode cullMode;

    void setProgramSource(const QQuickShaderEffectMaterialKey &source);
    void updateTextures() const;
    void invalidateTextureProvider(QSGTextureProvider *provider);

protected:
    friend class QQuickCustomMaterialShader;

    // The type pointer needs to be unique. It is not safe to let the type object be part of the
    // QQuickShaderEffectMaterial, since it can be deleted and a new one constructed on top of the old
    // one. The new QQuickShaderEffectMaterial would then get the same type pointer as the old one, and
    // CustomMaterialShaders based on the old one would incorrectly be used together with the new
    // one. To guarantee that the type pointer is unique, the type object must live as long as
    // there are any CustomMaterialShaders of that type.
    QSharedPointer<QSGMaterialType> m_type;
    QQuickShaderEffectMaterialKey m_source;

    QQuickShaderEffectNode *m_node;
    bool m_emittedLogChanged;

    static QHash<QQuickShaderEffectMaterialKey, QSharedPointer<QSGMaterialType> > materialMap;
};


class QSGShaderEffectMesh;

class Q_QUICK_PRIVATE_EXPORT QQuickShaderEffectNode : public QObject, public QSGGeometryNode
{
    Q_OBJECT
public:
    QQuickShaderEffectNode();
    virtual ~QQuickShaderEffectNode();

    virtual void preprocess();

Q_SIGNALS:
    void logAndStatusChanged(const QString &, int status);

private Q_SLOTS:
    void markDirtyTexture();
    void textureProviderDestroyed(QObject *object);
};

QT_END_NAMESPACE

QT_END_HEADER

#endif // QQUICKSHADEREFFECTNODE_P_H
